"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
var __param = (this && this.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const common_1 = require("@nestjs/common");
const axios_1 = __importDefault(require("axios"));
const nestjs_event_store_constant_1 = require("./nestjs-event-store.constant");
let EventStore = class EventStore {
    constructor(eventStore, configService, esStreamConfig) {
        this.category = esStreamConfig.name;
        this.resolveLinkTos = esStreamConfig.resolveLinkTos;
        this.eventStore = eventStore;
        this.eventStore.connect(configService.tcp);
        this.eventStoreHostUrl = configService.http.protocol +
            `://${configService.tcp.hostname}:${configService.http.port}/streams/`;
    }
    async publish(event) {
        if (event === undefined) {
            return;
        }
        if (event === null) {
            return;
        }
        const category = Object.keys(event)[0];
        const categoryId = event[category].id;
        const streamName = `${category}-${categoryId}`;
        const type = event.constructor.name;
        try {
            await this.eventStore.client.writeEvent(streamName, type, event);
        }
        catch (err) {
            common_1.Logger.error(err, err, 'EventStore');
        }
    }
    async bridgeEventsTo(subject) {
        if (Array.isArray(this.category)) {
            return;
        }
        const streamName = `$ce-${this.category}`;
        const onEvent = async (event) => {
            if (event.isJson && typeof event.data !== 'string') {
                const eventType = event.eventType;
                const data = event.data;
                event = this.eventHandlers[eventType](...Object.values(data));
                subject.next(event);
            }
            else {
                const eventUrl = `${this.eventStoreHostUrl}${event.metadata.$o}/${event.data.split('@')[0]}?embed=body`;
                return axios_1.default.get(eventUrl, {
                    responseType: 'stream',
                    headers: { Accept: 'application/vnd.eventstore.atom+json' },
                }).then((response) => {
                    const stream = response.data;
                    let rawData = '';
                    stream.on('data', (chunk) => { rawData += chunk; });
                    stream.on('end', () => {
                        const temp = JSON.parse(rawData);
                        const data = temp.content.data;
                        const eventType = temp.content.eventType;
                        event = this.eventHandlers[eventType](...Object.values(data));
                        subject.next(event);
                    });
                });
            }
        };
        const onDropped = (subscription, reason, error) => {
            common_1.Logger.error(error, reason, subscription);
        };
        try {
            if (!Array.isArray(this.category)) {
                await this.eventStore.client.subscribeToStream(streamName, onEvent, onDropped, this.resolveLinkTos);
            }
        }
        catch (err) {
            common_1.Logger.error(err, err, 'EventStore');
        }
    }
    setEventHandlers(eventHandlers) {
        this.eventHandlers = eventHandlers;
    }
};
EventStore = __decorate([
    common_1.Injectable(),
    __param(0, common_1.Inject(nestjs_event_store_constant_1.ProvidersConstants.EVENT_STORE_PROVIDER)),
    __param(1, common_1.Inject(nestjs_event_store_constant_1.ProvidersConstants.EVENT_STORE_CONNECTION_CONFIG_PROVIDER)),
    __param(2, common_1.Inject(nestjs_event_store_constant_1.ProvidersConstants.EVENT_STORE_STREAM_CONFIG_PROVIDER)),
    __metadata("design:paramtypes", [Object, Object, Object])
], EventStore);
exports.EventStore = EventStore;
//# sourceMappingURL=event-store.js.map